@@ -3,22 +3,25 @@ import re
 import sys
 import os
 import subprocess
+import time
+
 from . import genera
 from ._version import __version__
 
-__all__ = ['lifelib_dir', 'compile_rules', 'load_rules', 'reset_tree']
+from .genera.rulefiles import rule2files
+
+__all__ = ['lifelib_dir', 'compile_rules', 'load_rules', 'reset_tree',
+           'rule2files', 'run_tests', 'add_cygdir', 'install_cygwin']
 
 lifelib_dir = os.path.dirname(os.path.abspath(__file__))
 this_python = os.path.abspath(sys.executable)
 cygwin_dirs = []
 
-
 def add_cygdir(*args):
 
     for a in args:
         cygwin_dirs.append(a)
 
-
 def get_cygwin_dir():
 
     cygwin_possibilities = cygwin_dirs + [os.path.join(lifelib_dir, 'cygwin', 'cygwin64'), 'C:\\cygwin64']
@@ -29,7 +32,6 @@ def get_cygwin_dir():
 
     raise ValueError("Cygwin directory unknown; please call lifelib.add_cygdir(r'D:\\path\\to\\cygwin64') or lifelib.install_cygwin()")
 
-
 def install_cygwin(root_directory=None, packages=['python2', 'python2-numpy', 'python3', 'python3-numpy', 'gcc-g++', 'make', 'git']):
     '''
     For lifelib to run correctly on Windows, there needs to be a 64-bit Cygwin
@@ -110,7 +112,11 @@ def verify_installation():
     subprocess.check_call(get_local_bash() + ['echo', 'SUCCESS'])
 
     print("Verifying numpy is installed...")
-    np_version = subprocess.check_output(get_local_bash() + ['python', '-c', 'import numpy as np; print(np.__version__)'])
+    if (os.name == 'nt'):
+        np_version = subprocess.check_output(get_local_bash() + ['python', '-c', 'import numpy as np; print(np.__version__)'])
+    else:
+        import numpy as np
+        np_version = np.__version__
     np_version = regstring(np_version).strip()
     print("Numpy version: %s" % np_version)
     return np_version
@@ -148,6 +154,20 @@ def run_tests():
     print("Completed tests successfully.")
 
 
+def obtain_arguments(soname, standard="C++11", optimisations=["-march=native", "-O3"],
+                    options=["-Wall", "-Wextra"], **unused_kwargs):
+
+    so_args = ["-fPIC", "-shared", "-o", soname, "lifelib.cpp"]
+
+    if isinstance(standard, str):
+        standard = standard.lower()
+        if standard[0] != '-':
+            standard = "-std=" + standard
+        standard = [standard]
+
+    return standard + optimisations + options + so_args
+
+
 def compile_rules(*rules, **kwargs):
 
     if (len(rules) == 0):
@@ -180,6 +200,21 @@ def compile_rules(*rules, **kwargs):
 
     if recompile:
 
+        etime = 20.0
+
+        try:
+            with open(os.path.join(lifelib_dir, 'compiletime.txt'), 'r') as f:
+                for l in f:
+                    l = l.strip()
+                    if '.' in l:
+                        etime = float(l)
+        except:
+            pass
+
+
+        pbar = None
+        imax = 64
+
         try:
 
             cwd = os.path.abspath(os.getcwd())
@@ -188,14 +223,42 @@ def compile_rules(*rules, **kwargs):
             generate_code(rules)
             os.chdir(lifelib_dir)
 
-            print("Compiling lifelib shared object (this may take a while)...")
-            po = subprocess.Popen(get_compiler() + ["-std=c++11", "-march=native", "-O3",
-                "-Wall", "-Wextra", "-fPIC", "-shared", "-o", soname, "lifelib.cpp"],
-                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            print("Compiling lifelib shared object (estimated time: %.1f seconds)..." % etime)
+            ccargs = get_compiler() + obtain_arguments(soname, **kwargs)
+            po = subprocess.Popen(ccargs, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+            i = 0
+
+            try:
+                from tqdm.auto import tqdm
+                pbar = tqdm(total=100, desc='Estimated compilation progress: ')
+                imax = 99
+            except:
+                sys.stdout.write('[' + (' ' * imax) + ']\r[')
+
+            while po.poll() is None:
+                time.sleep(etime / imax)
+                if (i < imax):
+                    if pbar is None:
+                        sys.stdout.write('=')
+                        sys.stdout.flush()
+                    else:
+                        pbar.update(1)
+                i += 1
 
             g_stdout, g_stderr = po.communicate()
             status = po.returncode
 
+            if pbar is None:
+                char = '=' if (status == 0) else '!'
+                if (i < imax):
+                    sys.stdout.write((char * (imax - i)))
+                sys.stdout.write(']\n')
+                sys.stdout.flush()
+            elif (status == 0):
+                for j in range(100 - min(imax, i)):
+                    pbar.update(1)
+
             if kwargs.get('compiler_output', (status != 0)):
                 sys.stderr.write("Compiler stdout: %s\n" % g_stdout)
                 sys.stderr.write("Compiler stderr: %s\n" % g_stderr)
@@ -205,8 +268,17 @@ def compile_rules(*rules, **kwargs):
 
         finally:
 
+            if pbar is not None:
+                pbar.close()
             os.chdir(cwd)
 
+        etime = ((etime * i) / imax) + 0.5
+
+        print('...compilation complete!')
+
+        with open(os.path.join(lifelib_dir, 'compiletime.txt'), 'w') as f:
+            f.write(str(etime))
+
         reset_tree()
             
     return soname
@@ -214,7 +286,13 @@ def compile_rules(*rules, **kwargs):
 
 def load_rules(*rules, **kwargs):
 
-    rules = [r.lower() for r in rules]
+    rules = list(rules)
+
+    for i in range(len(rules)):
+        if '.' in rules[i]:
+            rules[i] = rule2files(rules[i])
+        else:
+            rules[i] = rules[i].lower()
 
     soname = compile_rules(*rules, **kwargs)
     from .pythlib import Session
@@ -224,12 +302,31 @@ def load_rules(*rules, **kwargs):
 
     # Choose the default number of planes to accommodate every rule
     # for which the lifetree has been compiled, including LifeHistory:
-    default_planes = 1 + max([genera.rule_property(r, 'bitplanes') for r in rules])
+    default_planes = max([genera.rule_property(r, 'bitplanes') for r in rules])
     default_planes = max(4, default_planes)
 
     return Session(soname, rules=rules, local_bash=local_bash,
                     local_python=local_python, default_planes=default_planes)
 
+def write_grid_iterator(f, rules, families):
+
+    f.write('    void iterate_var_grid(int rule, uint8_t *ingrid, uint8_t *outgrid) {\n')
+    f.write('        switch(rule) {\n')
+    not_used = True
+    for (i, (runsafe, family)) in enumerate(zip(rules, families)):
+        r = runsafe.replace('-', '_')
+
+        if (family == 6) or (family == 8):
+            not_used = False
+            f.write('            case %d :\n' % i)
+            f.write('                %s::iterate_var_grid(ingrid, outgrid); break;\n' % r)
+
+    f.write('        }\n')
+    if not_used:
+        f.write('        (void) ingrid; (void) outgrid;\n')
+    else:
+        f.write('        #define LIFELIB_FAMILY_SIX 1\n')
+    f.write('    }\n\n')
 
 def write_all_iterators(f, hist, rules, families):
 
@@ -272,7 +369,7 @@ def write_all_iterators(f, hist, rules, families):
             not_used = False
             f.write('            case %d :\n' % i)
             f.write('                return %s::iterate_var_leaf(inleaves, hleaves, outleaf);\n' % r)
-        elif ((family != 2) and (hist == 0)):
+        elif ((family == 4) and (hist == 0)):
             not_used = False
             f.write('            case %d :\n' % i)
             f.write('                return %s::iterate_var_leaf(inleaves, outleaf);\n' % r)
@@ -314,8 +411,11 @@ def reset_tree(rule='b3s23'):
         cwd = os.path.abspath(os.getcwd())
         generate_code([rule], clean_before=True)
     except:
-        import warnings
-        warnings.warn("Could not reset tree -- check permissions?")
+        if (rule != "b3s23"):
+            raise
+        else:
+            import warnings
+            warnings.warn("Could not reset tree -- check permissions?")
     finally:
         os.chdir(cwd)
 
@@ -388,6 +488,8 @@ def generate_code(rules, clean_before=False):
         for hist in range(3):
             write_all_iterators(f, hist, rules, families)
 
+        write_grid_iterator(f, rules, families)
+
         f.write('}\n')
 
     # Create the source code for the rules:
