@@ -16,3 +16,8 @@ genus_list.append({'name': 'isogeny', 'regex': 'g[1-9][0-9]*b[1-6omp-]*s[0-6omp-
 genus_list.append({'name': 'bsfkl', 'regex': 'b1?2?3?4?5?6?7?8?s0?1?2?3?4?5?6?7?8?f0?1?2?3?4?5?6?7?8?k0?1?2?3?4?5?6?7?8?l0?1?2?3?4?5?6?7?8?'})
 genus_list.append({'name': 'hrot', 'regex': 'r[2345]b[0-9a-f]*s[0-9a-f]*z?'})
 genus_list.append({'name': 'ghrot', 'regex': 'g[1-9][0-9]*r[2345]b[0-9a-f]*s[0-9a-f]*z?'})
+
+# This one must always be last as a 'catch-all' for custom rules with
+# no more than 256 states:
+
+genus_list.append({'name': 'eightbit', 'regex': 'x[0-9a-z_-]*'})
