@@ -65,7 +65,7 @@ def str2tab(rulestring, hexagonal='unknown'):
 
         import re
 
-        regex = '1?(2[omp-]*)?(3[omp-]*)?(4[omp-]*)?5?6?'
+        regex = '1?(2([-]?[omp])?)?(3([-]?[omp])?)?(4([-]?[omp])?)?5?6?'
         regex = '^b' + regex + 's0?' + regex + 'h$'
 
         if not re.match(regex, rulestring):
