@@ -45,7 +45,10 @@ class SampleSoupList(Sequence):
     def __repr__(self):
         return 'SampleSoupList(%r, %r, %r, %r)' % (self.lt, self.rule, self.symmetry, self.samples)
 
-def download_samples(apgcode, rule, lt, domain='https://gol.hatsya.co.uk', tempfile='tempfile'):
+def download_samples(lt, apgcode, rule, domain='https://gol.hatsya.co.uk', tempfile='tempfile'):
+
+    if ('_' in rule) and ('_' not in apgcode):
+        rule, apgcode = apgcode, rule
 
     try:
         from urllib import urlretrieve
