@@ -53,9 +53,9 @@ class Lifetree(object):
         pptr = self.lifelib('Hashsoup', self.ptr, rule, symmetry, seed)
         return Pattern(self.session, pptr, self)
 
-    def download_samples(self, rule, apgcode, **kwargs):
+    def download_samples(self, apgcode, rule, **kwargs):
 
-        return dsamples(apgcode, rule, self, **kwargs)
+        return dsamples(self, apgcode, rule, **kwargs)
 
     def pattern(self, rle="", rule=None, tempfile='tempfile', verify_apgcode=True):
 
